==> home-made-io-fastmag-sync-m2-0.21.1-3b0020.zip: Process/Manager/ToMagento.php <==
<?php
/*******************************************************************************
 * Created by HOMEMADE.IO SAS.
 * Fastmag Sync  -  Connect Fastmag with your Magento
 *
 * Copyright (C) HOMEMADE.IO SAS, Inc - All Rights Reserved
 *
 * @author    Simon Laubet-Xavier <simon.laubetxavier@home-made.io>
 * @copyright 2020-2021 HOMEMADE.IO SAS
 * @date      2021-09-06
 ******************************************************************************/

namespace Fastmag\Sync\Process\Manager;

use Exception;
use Fastmag\Sync\Api\Data\Jobqueue\ToMagentoInterface as Job;
use Fastmag\Sync\Exception\ProcessException;
use Fastmag\Sync\Logger\Logger;
use Fastmag\Sync\Model\Config;
use Fastmag\Sync\Model\Jobqueue as AbstractJob;
use Fastmag\Sync\Model\Jobqueue\ToMagentoRepository as JobRepository;
use Fastmag\Sync\Model\ResourceModel\Jobqueue\ToMagento\Collection;
use Fastmag\Sync\Process\Manager;
use Fastmag\Sync\Process\Worker\ToMagento\Clean;
use Fastmag\Sync\Process\Worker\ToMagento\Integration;
use Fastmag\Sync\Process\Worker\ToMagento\Integration\Product\Inventory;
use Fastmag\Sync\Process\Worker\ToMagento\Scheduled;
use Fastmag\Sync\Process\WorkerFactory;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Api\SortOrderBuilder;
use Magento\Framework\App\Area;
use Magento\Framework\App\State;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\LocalizedException;

/**
 * Class ToMagento
 *
 * Workers manager for synchronization from Fastmag to Magento
 */
class ToMagento extends Manager
{
    /** @var SearchCriteriaBuilder $searchCriteriaBuilder */
    protected $searchCriteriaBuilder;

    /** @var SortOrderBuilder $sortOrderBuilder */
    protected $sortOrderBuilder;

    /** @var JobRepository $jobRepository */
    protected $jobRepository;

    /** @var State $state */
    protected $state;

    /**
     * ToMagento constructor
     *
     * @param Logger                $logger
     * @param Config                $config
     * @param WorkerFactory         $workerFactory
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param SortOrderBuilder      $sortOrderBuilder
     * @param JobRepository         $jobRepository
     * @param State                 $state
     */
    public function __construct(
        Logger $logger,
        Config $config,
        WorkerFactory $workerFactory,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        SortOrderBuilder $sortOrderBuilder,
        JobRepository $jobRepository,
        State $state
    ) {
        parent::__construct($logger, $config, $workerFactory);

        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->sortOrderBuilder = $sortOrderBuilder;
        $this->jobRepository = $jobRepository;
        $this->state = $state;
    }

    /**
     * Main method
     *
     * @param string|null $jobCode
     * @param string|null $jobId
     *
     * @return void
     *
     * @throws ProcessException
     */
    public function run($jobCode = null, $jobId = null)
    {
        try {
            $checkOk = $this->checkPrerequisites($jobCode, $jobId);

            if ($checkOk) {
                $this->logger->debug('To Magento Sync - Begin process');

                $this->state->setAreaCode(Area::AREA_ADMINHTML);

                $this->moveScheduledJobs();

                $jobsCollectionsByPriorityCode = $this->getJobsCollections($jobCode, $jobId);

                if (count($jobsCollectionsByPriorityCode) > 0) {
                    foreach ($jobsCollectionsByPriorityCode as $collectionPriority => $jobsCollectionByPriorityCode) {
                        /** @var Collection $jobsCollection */
                        foreach ($jobsCollectionByPriorityCode as $collectionJobCode => $jobsCollection) {
                            $this->processJobs($collectionJobCode, $jobsCollection);
                        }
                    }
                }

                /** @var Clean cleanWorker */
                $cleanWorker = $this->workerFactory->create('tomagento_clean');
                $cleanWorker->run();
            }
        } catch (ProcessException $exception) {
            $this->logger->critical($exception->getMessage());

            throw $exception;
        } catch (LocalizedException $exception) {
            $this->logger->critical($exception->getMessage());

            throw new ProcessException(__($exception->getMessage()));
        }
    }

    /**
     * Check required config fields to run the synchronization
     *
     * @param string|null $jobCode
     * @param string|null $jobId
     *
     * @return bool
     *
     * @throws ProcessException
     */
    protected function checkPrerequisites($jobCode = null, $jobId = null)
    {
        $result = parent::checkPrerequisites();

        if ($result) {
            if ($jobId !== null) {
                try {
                    $job = $this->jobRepository->getById($jobId);
                    $jobCode = $job->getJobCode();
                } catch (Exception $exception) {
                    throw new ProcessException(__('Job ID #%1 does not exist in queue.', $jobId));
                }
            }

            if ($jobCode !== null) {
                try {
                    /** @var Integration $worker */
                    $worker = $this->workerFactory->create($jobCode);
                } catch (Exception $exception) {
                    throw new ProcessException(__('There\'s no worker with code "%1".', $jobCode));
                }

                if (!$worker->isEnabled()) {
                    throw new ProcessException(__('Jobs "%1" are not enabled.', $jobCode));
                }
            }
        }

        return $result;
    }

    /**
     * Run the specific worker to move scheduled jobs into instant queue
     *
     * @return void
     *
     * @throws ProcessException
     */
    protected function moveScheduledJobs()
    {
        if ($this->config->isSetFlag(Config::XML_PATH_PRODUCT_PRICE_MANAGE_TIER_PRICES)) {
            /** @var Scheduled $scheduledWorker */
            $scheduledWorker = $this->workerFactory->create('tomagento_move_scheduled');
            $scheduledWorker->run();
        }
    }

    /**
     * Get array of jobs collection to process
     *
     * @param string|null $jobCode
     * @param string|null $jobId
     *
     * @return array
     */
    protected function getJobsCollections($jobCode, $jobId)
    {
        $sortOrderPriority = $this->sortOrderBuilder->setField(Job::PRIORITY)
            ->setDescendingDirection()
            ->create();
        $sortOrderJobId = $this->sortOrderBuilder->setField(Job::JOB_ID)
            ->setAscendingDirection()
            ->create();

        $this->searchCriteriaBuilder->addFilter(Job::STATUS, AbstractJob::STATUS_OK)
            ->addFilter(Job::PROCESSED_AT, null, 'null')
            ->setSortOrders([$sortOrderPriority, $sortOrderJobId])
            ->setPageSize($this->getSyncLimit());

        if ($jobId) {
            $this->searchCriteriaBuilder->addFilter(Job::JOB_ID, $jobId);
        } elseif ($jobCode) {
            $this->searchCriteriaBuilder->addFilter(Job::JOB_CODE, $jobCode);
        }

        $searchCriteria = $this->searchCriteriaBuilder->create();

        return $this->jobRepository->getListByPriorityAndCode($searchCriteria);
    }

    /**
     * Process jobs collection
     *
     * @param string     $jobCode
     * @param Collection $jobsCollection
     *
     * @return void
     *
     * @throws ProcessException
     */
    protected function processJobs($jobCode, $jobsCollection)
    {
        if ($jobsCollection->count() > 0) {
            $worker = $this->getJobsWorker($jobCode);

            if ($worker->isEnabled()) {
                $this->logger->debug(__('[%1] Beginning hydration', $this->getHydrationJobCode($jobCode))->render());

                $this->hydrateJobs($jobCode, $jobsCollection);
                $this->logger->debug(
                    __('[%1] Hydration done, beginning integration.', $worker->getCode())->render()
                );

                $this->runWorker($worker, $jobsCollection);
                $this->logger->debug(__('[%1] Integration done', $worker->getCode())->render());

                $this->validateJobs($jobsCollection, $worker->getCode());
            } else {
                $this->logger->debug(
                    __('[%1] Workers for jobs "%2" are not enabled.', $worker->getCode(), $jobCode)->render()
                );
            }
        }
    }

    /**
     * Return worker instance for the current job code
     *
     * @param string $jobCode
     *
     * @return Integration
     */
    protected function getJobsWorker($jobCode)
    {
        if (strpos($jobCode, 'tomagento_integration_product_') === 0
            && $this->config->isSetFlag(Config::XML_PATH_PRODUCT_IMPORT_ONLY_INVENTORY)
        ) {
            $this->logger->debug(
                'Jobs ' . $jobCode . ' - "Stock only" config field set, '
                . 'switched to tomagento_integration_product_quantity worker'
            );

            /** @var Inventory $result */
            $result = $this->workerFactory->create('tomagento_integration_product_quantity');
        } else {
            /** @var Integration $result */
            $result = $this->workerFactory->create($jobCode);
        }

        return $result;
    }

    /**
     * Get hydration worker code for sync job code
     *
     * @todo make this method more generic
     *
     * @param string $jobCode
     *
     * @return string
     */
    protected function getHydrationJobCode($jobCode)
    {
        $result = '';

        if (stripos($jobCode, 'product') !== false) {
            if (stripos($jobCode, 'quantity') !== false) {
                $result = 'tomagento_hydration_product_inventory';
            } elseif (stripos($jobCode, 'price') !== false) {
                $result = 'tomagento_hydration_product_price';
            } else {
                $result = 'tomagento_hydration_product';
            }
        } elseif (stripos($jobCode, 'order') !== false) {
            $result = 'tomagento_hydration_order';
        }

        return $result;
    }

    /**
     * Validate and save jobs
     *
     * @param Collection $jobsCollection
     * @param string     $workerCode
     *
     * @return void
     */
    protected function validateJobs($jobsCollection, $workerCode)
    {
        if ($jobsCollection->count() > 0) {
            foreach ($jobsCollection->getItems() as $job) {
                if (!$job->isInError()) {
                    $this->jobRepository->validate($job);
                    try {
                        $this->jobRepository->save($job);
                    } catch (CouldNotSaveException $exception) {
                        $this->logger->critical(
                            '[' . $workerCode . '][Job #' . $job->getId() . '][Entity ' . $job->getContentId() . '] '
                            . '[ERROR] Unable to save job.'
                        );
                    }

                    $this->logger->debug(
                        '[' . $workerCode . '][Job #' . $job->getId() . '][Entity ' . $job->getContentId() . '] '
                        . 'Job validated'
                    );
                }
            }
        }
    }
}

==> home-made-io-fastmag-sync-m2-0.24.0-762a73.zip: Process/Manager/ToMagento.php <==
<?php
/*******************************************************************************
 * Created by HOMEMADE.IO SAS.
 * Fastmag Sync  -  Connect Fastmag with your Magento
 *
 * Copyright (C) HOMEMADE.IO SAS, Inc - All Rights Reserved
 *
 * @author    Simon Laubet-Xavier <simon.laubetxavier@home-made.io>
 * @copyright 2020-2021 HOMEMADE.IO SAS
 * @date      2021-09-30
 ******************************************************************************/

namespace Fastmag\Sync\Process\Manager;

use Exception;
use Fastmag\Sync\Api\Data\Jobqueue\ToMagentoInterface as Job;
use Fastmag\Sync\Exception\ProcessException;
use Fastmag\Sync\Logger\Logger;
use Fastmag\Sync\Model\Config;
use Fastmag\Sync\Model\Jobqueue as AbstractJob;
use Fastmag\Sync\Model\Jobqueue\ToMagentoRepository as JobRepository;
use Fastmag\Sync\Model\ResourceModel\Jobqueue\ToMagento\Collection as JobCollection;
use Fastmag\Sync\Process\Manager;
use Fastmag\Sync\Process\Worker\ToMagento\Clean;
use Fastmag\Sync\Process\Worker\ToMagento\Integration;
use Fastmag\Sync\Process\Worker\ToMagento\Integration\Product\Inventory;
use Fastmag\Sync\Process\Worker\ToMagento\Scheduled;
use Fastmag\Sync\Process\WorkerFactory;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Api\SortOrderBuilder;
use Magento\Framework\Exception\LocalizedException;

/**
 * Class ToMagento
 *
 * Workers manager for synchronization from Fastmag to Magento
 */
class ToMagento extends Manager
{
    /** @var SearchCriteriaBuilder $searchCriteriaBuilder */
    protected $searchCriteriaBuilder;

    /** @var SortOrderBuilder $sortOrderBuilder */
    protected $sortOrderBuilder;

    /** @var JobRepository $jobRepository */
    protected $jobRepository;

    /**
     * ToMagento constructor
     *
     * @param Logger                $logger
     * @param Config                $config
     * @param WorkerFactory         $workerFactory
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param SortOrderBuilder      $sortOrderBuilder
     * @param JobRepository         $jobRepository
     */
    public function __construct(
        Logger $logger,
        Config $config,
        WorkerFactory $workerFactory,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        SortOrderBuilder $sortOrderBuilder,
        JobRepository $jobRepository
    ) {
        parent::__construct($logger, $config, $workerFactory);

        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->sortOrderBuilder = $sortOrderBuilder;
        $this->jobRepository = $jobRepository;
    }

    /**
     * @inheritDoc
     *
     * @return JobRepository
     */
    protected function getJobRepository()
    {
        return $this->jobRepository;
    }

    /**
     * @inheritDoc
     *
     * @return JobCollection
     */
    protected function getCurrentJobsCollection()
    {
        return $this->currentJobsCollection;
    }

    /**
     * Main method
     *
     * @param string|null $jobCode
     * @param string|null $jobId
     *
     * @return void
     *
     * @throws ProcessException
     */
    public function run($jobCode = null, $jobId = null)
    {
        try {
            $checkOk = $this->checkPrerequisites($jobCode, $jobId);

            if ($checkOk) {
                $this->logger->debug('To Magento Sync - Begin process');

                $this->moveScheduledJobs();

                $jobsCollectionsByPriorityCode = $this->getJobsCollections($jobCode, $jobId);

                if (count($jobsCollectionsByPriorityCode) > 0) {
                    foreach ($jobsCollectionsByPriorityCode as $collectionPriority => $jobsCollectionByPriorityCode) {
                        foreach ($jobsCollectionByPriorityCode as $collectionJobCode => $jobsCollection) {
                            $this->processJobs($collectionJobCode, $jobsCollection);
                        }
                    }
                }

                /** @var Clean cleanWorker */
                $cleanWorker = $this->workerFactory->create('tomagento_clean');
                $cleanWorker->run();
            }
        } catch (ProcessException $exception) {
            $this->logger->critical($exception->getMessage());

            throw $exception;
        } catch (LocalizedException $exception) {
            $this->logger->critical($exception->getMessage());

            throw new ProcessException(__($exception->getMessage()));
        }
    }

    /**
     * Check required config fields to run the synchronization
     *
     * @param string|null $jobCode
     * @param string|null $jobId
     *
     * @return bool
     *
     * @throws ProcessException
     */
    protected function checkPrerequisites($jobCode = null, $jobId = null)
    {
        $result = parent::checkPrerequisites();

        if ($result) {
            if ($jobId !== null) {
                try {
                    $job = $this->getJobRepository()->getById($jobId);

                    $jobCode = $job->getJobCode();
                } catch (Exception $exception) {
                    throw new ProcessException(__('Job ID #%1 does not exist in queue.', $jobId));
                }
            }

            if ($jobCode !== null) {
                try {
                    /** @var Integration $worker */
                    $worker = $this->workerFactory->create($jobCode);
                } catch (Exception $exception) {
                    throw new ProcessException(__('There\'s no worker with code "%1".', $jobCode));
                }

                if (!$worker->isEnabled()) {
                    throw new ProcessException(__('Jobs "%1" are not enabled.', $jobCode));
                }
            }
        }

        return $result;
    }

    /**
     * Run the specific worker to move scheduled jobs into instant queue
     *
     * @return void
     *
     * @throws ProcessException
     */
    protected function moveScheduledJobs()
    {
        if ($this->config->isSetFlag(Config::XML_PATH_PRODUCT_PRICE_MANAGE_TIER_PRICES)) {
            /** @var Scheduled $scheduledWorker */
            $scheduledWorker = $this->workerFactory->create('tomagento_move_scheduled');
            $scheduledWorker->run();
        }
    }

    /**
     * Get array of jobs collection to process
     *
     * @param string|null $jobCode
     * @param string|null $jobId
     *
     * @return array
     */
    protected function getJobsCollections($jobCode, $jobId)
    {
        $sortOrderPriority = $this->sortOrderBuilder->setField(Job::PRIORITY)
            ->setDescendingDirection()
            ->create();
        $sortOrderJobId = $this->sortOrderBuilder->setField(Job::JOB_ID)
            ->setAscendingDirection()
            ->create();

        $this->searchCriteriaBuilder->addFilter(Job::STATUS, AbstractJob::STATUS_PENDING)
            ->setSortOrders([$sortOrderPriority, $sortOrderJobId])
            ->setPageSize($this->getSyncLimit());

        if ($jobId) {
            $this->searchCriteriaBuilder->addFilter(Job::JOB_ID, $jobId);
        } elseif ($jobCode) {
            $this->searchCriteriaBuilder->addFilter(Job::JOB_CODE, $jobCode);
        }

        $searchCriteria = $this->searchCriteriaBuilder->create();

        return $this->getJobRepository()->getListByPriorityAndCode($searchCriteria);
    }

    /**
     * Return worker instance for the current job code
     *
     * @param string $jobCode
     *
     * @return Integration
     */
    protected function getJobsWorker($jobCode)
    {
        if (strpos($jobCode, 'tomagento_integration_product_') === 0
            && $this->config->isSetFlag(Config::XML_PATH_PRODUCT_IMPORT_ONLY_INVENTORY)
        ) {
            $this->logger->debug(
                'Jobs ' . $jobCode . ' - "Stock only" config field set, '
                . 'switched to tomagento_integration_product_quantity worker'
            );

            /** @var Inventory $result */
            $result = $this->workerFactory->create('tomagento_integration_product_quantity');
        } else {
            $result = parent::getJobsWorker($jobCode);
        }

        return $result;
    }
}
